# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaMess -- Fabric Automation Helpers
#  Copyright © 2024 Lance Edgar
#
#  This file is part of Wutta Framework.
#
#  Wutta Framework is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option) any
#  later version.
#
#  Wutta Framework is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  Wutta Framework.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Misc. Utilities
"""

from pathlib import Path
from typing_extensions import Any, Mapping

from mako.template import Template


def exists(c, path):
    """
    Returns ``True`` if given path exists on the host, otherwise ``False``.
    """
    return not c.run(f'test -e {path}', warn=True).failed


def get_home_path(c, user=None):
    """
    Get the path to user's home folder on target machine.

    :param c: Fabric connection.

    :param user: Username whose home folder you want.  If not
       specified, the username for the current connection is assumed.

    :returns: Home folder path as string.
    """
    user = user or c.user
    home = c.run(f'getent passwd {user} | cut -d: -f6').stdout.strip()
    home = home.rstrip('/')
    return home


def is_symlink(c, path):
    """
    Check if the given path is a symlink.

    :param c: Fabric connection.

    :param path: Path to check, on target machine.

    :returns: ``True`` if path is a symlink, else ``False``.
    """
    # nb. this function is derived from one copied from fabric v1
    cmd = 'test -L "$(echo %s)"' % path
    result = c.run(cmd, warn=True)
    return False if result.failed else True


def mako_renderer(c, env={}):
    """
    This returns a *function* suitable for use as a ``fabsync`` file
    renderer.  The function assumes the file is a Mako template.

    :param c: Fabric connection.

    :param env: Environment dictionary to be used as Mako template
       context.

    Typical usage is something like::

       from fabric import task
       from wuttamess import sync, util

       root = sync.make_root('files')
       env = {}

       @task
       def foo(c):

           # define possible renderers for fabsync
           renderers = {'mako': util.mako_renderer(c, env)}

           sync.check_isync(c, root, 'etc/postfix', renderers=renderers)
    """
    def render(path: Path, vars: Mapping[str, Any], **kwargs) -> bytes:
        return Template(filename=str(path)).render(**env)

    return render


def set_timezone(c, timezone):
    """
    Set the system timezone.

    :param c: Fabric connection.

    :param timezone: Standard timezone name,
       e.g. ``'America/Chicago'``.
    """
    c.run(f"bash -c 'echo {timezone} > /etc/timezone'")

    if is_symlink(c, '/etc/localtime'):
        c.run(f'ln -sf /usr/share/zoneinfo/{timezone} /etc/localtime')
    else:
        c.run(f'cp /usr/share/zoneinfo/{timezone} /etc/localtime')
